/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: NotificationSender.java,v 1.14 2007/04/02 23:24:08 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.jython.lib;

import java.net.URI;

import org.python.core.PyObject;
import org.xmlpull.infoset.XmlElement;

import wsmg.WseClientAPI;
import xsul.XmlConstants;
import xsul.ws_addressing.WsaEndpointReference;
import xsul5.MLogger;
import edu.indiana.extreme.lead.workflow_tracking.Notifier;
import edu.indiana.extreme.lead.workflow_tracking.NotifierFactory;
import edu.indiana.extreme.lead.workflow_tracking.common.AnnotationProps;
import edu.indiana.extreme.lead.workflow_tracking.common.ConstructorConsts;
import edu.indiana.extreme.lead.workflow_tracking.common.ConstructorProps;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationContext;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationEntity;
import edu.indiana.extreme.xbaya.util.StringUtil;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class NotificationSender {

    private static final MLogger logger = MLogger.getLogger();

    private Notifier notifier;

    private String brokerURL;

    private String topic;

    private URI workflowID;

    private InvocationEntity initiator;

    private InvocationEntity receiver;

    private InvocationContext invocationContext;

    private WsaEndpointReference eventSink;

    /**
     * Constructs a NotificationSender.
     * 
     * @param brokerURL
     * @param topic
     */
    public NotificationSender(URI brokerURL, String topic) {
        this(brokerURL.toString(), topic);
    }

    /**
     * Creates a NotificationSender.
     * 
     * @param brokerURL
     *            The location of notification brokerUrl.
     * @param topic
     *            The notification topic.
     */
    public NotificationSender(String brokerURL, String topic) {
        logger.entering(new Object[] { brokerURL, topic });
        this.topic = topic;
        this.brokerURL = brokerURL;
        this.workflowID = URI.create(StringUtil
                .convertToJavaIdentifier(this.topic));

        this.eventSink = WseClientAPI.createEndpointReference(this.brokerURL,
                this.topic);
        ConstructorProps props = ConstructorProps.newProps();
        props.set(ConstructorConsts.BROKER_EPR, XmlConstants.BUILDER
                .serializeToString(this.eventSink));
        AnnotationProps annotationProps = AnnotationProps.newProps();
        props.set(ConstructorConsts.ANNOTATIONS, annotationProps);

        this.notifier = NotifierFactory.createNotifier(props);

        URI initiatorWorkflowID = null;
        URI initiatorServiceID = URI.create("XBaya");
        String initiatorWorkflowNodeID = null;
        Integer initiatorWorkflowTimeStep = null;
        this.initiator = this.notifier.createEntity(initiatorWorkflowID,
                initiatorServiceID, initiatorWorkflowNodeID,
                initiatorWorkflowTimeStep);

        URI receiverWorkflowID = this.workflowID;
        URI receiverServiceID = this.workflowID;
        String receiverWorkflowNodeID = null;
        Integer receiverWorkflowTimeStep = null;
        this.receiver = this.notifier.createEntity(receiverWorkflowID,
                receiverServiceID, receiverWorkflowNodeID,
                receiverWorkflowTimeStep);
    }

    /**
     * @return The event sink EPR.
     */
    public WsaEndpointReference getEventSink() {
        return this.eventSink;
    }

    /**
     * @param args
     * @param keywords
     */
    public void workflowStarted(PyObject[] args, String[] keywords) {
        logger.entering(new Object[] { args, keywords });
        String message = "";
        for (int i = 0; i < args.length; i++) {
            if (i != 0) {
                message += ", ";
            }
            message += keywords[i] + "=" + args[i];
        }
        this.invocationContext = this.notifier.workflowInvoked(this.receiver,
                this.initiator, message);
    }

    /**
     * @param args
     * @param keywords
     */
    public void workflowFinished(PyObject[] args, String[] keywords) {
        logger.entering(new Object[] { args, keywords });
        String message = "";
        for (int i = 0; i < args.length; i++) {
            if (i != 0) {
                message += ", ";
            }
            message += keywords[i] + "=" + args[i];
        }
        this.notifier.sendingResult(this.invocationContext, message);
        this.notifier.workflowTerminated(this.workflowID,
                "Workflow finished successfully.");
    }

    /**
     * Sends a START_INCOMPLETED notification message.
     * 
     * @param message
     *            The message to send
     */
    public void workflowFailed(String message) {
        workflowFailed(message, null);
    }

    /**
     * Sends a START_INCOMPLETED notification message.
     * 
     * @param e
     */
    public void workflowFailed(Throwable e) {
        workflowFailed(null, e);
    }

    /**
     * Sends a START_INCOMPLETED notification message.
     * 
     * @param message
     *            The message to send
     * @param e
     */
    public void workflowFailed(String message, Throwable e) {
        logger.entering(new Object[] { message, e });
        logger.caught(e);
        if (message == null || "".equals(message)) {
            message = "Error";
        }
        if (e != null) {
            message += ": " + e.toString();
        }
        if (e != null) {
            String stackTrace = StringUtil.getStackTraceInString(e);
            XmlElement stackTraceElement = XMLUtil.BUILDER
                    .newFragment("stackTrace");
            stackTraceElement.addChild(stackTrace);
            this.notifier.sendingFault(this.invocationContext, message, XMLUtil
                    .xmlElementToString(stackTraceElement));
        } else {
            this.notifier.sendingFault(this.invocationContext, message);
        }
    }

    /**
     * @param nodeID
     * @return The ServiceNoficationSender created.
     */
    public ServiceNotificationSender createServiceNotificationSender(
            String nodeID) {
        return new ServiceNotificationSender(this.notifier, this.eventSink,
                this.receiver, this.workflowID, nodeID);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
